"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = require("aws-sdk");
const base_1 = require("./base");
/**
 * DynamoDB Helper
 */
class DynamoDBHelper extends base_1.BaseHelper {
    /**
     * Initializes new instance of DynamoDBHelper
     * @param logger {ILogger} Injected logger
     * @param repository {AWS.DynamoDB.DocumentClient} Injected Repository. A new repository will be created if not supplied
     * @param options {AWS.DynamoDB.ClientConfiguration} Injected configuration if a Repository is supplied
     */
    constructor(logger, repository, options) {
        super(logger);
        this.Repository = repository || new AWS.DynamoDB.DocumentClient(options);
    }
    /**
     * Delete an item by key
     * @param tableName {string} Table name to delete from
     * @param keyName {string} Name of key column
     * @param keyValue {string | number} Value of key column
     */
    DeleteItemByKeyAsync(tableName, keyName, keyValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${DynamoDBHelper.name}.${this.DeleteItemByKeyAsync.name}`;
            this.TraceInputs(action, { tableName, keyName, keyValue });
            // guard clauses
            if (this.IsNullOrEmpty(tableName)) {
                throw new Error(`[${action}]-Must supply tableName`);
            }
            if (this.IsNullOrEmpty(keyName)) {
                throw new Error(`[${action}]-Must supply keyName`);
            }
            if (this.IsNullOrEmpty(keyValue.toString())) {
                throw new Error(`[${action}]-Must supply keyValue`);
            }
            // create attribute map
            const attributeMap = {};
            attributeMap[keyName] = keyValue;
            // create params object
            const params = {
                Key: attributeMap,
                TableName: tableName,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.delete(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    // /**
    //  * Delete an item by condition expression
    //  * @param tableName {string} Table name to delete from
    //  * @param expression
    //  * @param attributeNames
    //  * @param attributeValues
    //  */
    // public async DeleteItemByConditionAsync(tableName: string,
    //     expression: string,
    //     attributeNames: AWS.DynamoDB.DocumentClient.ExpressionAttributeNameMap,
    //     attributeValues: AWS.DynamoDB.DocumentClient.ExpressionAttributeValueMap): Promise<AWS.DynamoDB.DocumentClient.DeleteItemOutput> {
    //     const action = `${DynamoDBHelper.name}.${this.DeleteItemByKeyAsync.name}`;
    //     this.TraceInputs(action, { tableName, expression, attributeNames, attributeValues });
    //     // guard clauses
    //     if (this.IsNullOrEmpty(tableName)) { throw new Error(`[${action}]-Must supply tableName`); }
    //     if (this.IsNullOrEmpty(expression)) { throw new Error(`[${action}]-Must supply expression`); }
    //     if (this.IsNullOrEmpty(attributeNames)) { throw new Error(`[${action}]-Must supply attributeNames`); }
    //     if (this.IsNullOrEmpty(attributeValues)) { throw new Error(`[${action}]-Must supply attributeValues`); }
    //     // create params object
    //     const params: AWS.DynamoDB.DocumentClient.DeleteItemInput = {
    //         ConditionExpression: expression,
    //         ExpressionAttributeNames: attributeNames,
    //         ExpressionAttributeValues: attributeValues,
    //         TableName: tableName,
    //     };
    //     this.TraceRequest(action, params);
    //     // make AWS call
    //     const response = await this.Repository.delete(params).promise();
    //     this.TraceResponse(action, response);
    //     return response;
    // }
    /**
     * Get an item by key
     * @param tableName {string} Table name to get from
     * @param keyName {string} Name of key column
     * @param keyValue {string | number} Value of key column
     */
    GetItemByKeyAsync(tableName, keyName, keyValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${DynamoDBHelper.name}.${this.GetItemByKeyAsync.name}`;
            this.TraceInputs(action, { tableName, keyName, keyValue });
            // guard clauses
            if (this.IsNullOrEmpty(tableName)) {
                throw new Error(`[${action}]-Must supply tableName`);
            }
            if (this.IsNullOrEmpty(keyName)) {
                throw new Error(`[${action}]-Must supply keyName`);
            }
            if (this.IsNullOrEmpty(keyValue.toString())) {
                throw new Error(`[${action}]-Must supply keyValue`);
            }
            // create attribute map
            const attributeMap = {};
            attributeMap[keyName] = keyValue;
            // create params object
            const params = {
                Key: attributeMap,
                TableName: tableName,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.get(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Put an item
     * @param tableName {string} Table name to put item in
     * @param item {T extends object} Item to put
     */
    PutItemByKeyAsync(tableName, item) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${DynamoDBHelper.name}.${this.PutItemByKeyAsync.name}`;
            this.TraceInputs(action, { tableName, item });
            // guard clauses
            if (this.IsNullOrEmpty(tableName)) {
                throw new Error(`[${action}]-Must supply tableName`);
            }
            if (!item || Object.keys(item).length === 0) {
                throw new Error(`[${action}]-Must supply item`);
            }
            // create params object
            const params = {
                Item: item,
                TableName: tableName,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.put(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Scan a table
     * @param tableName {string} Table name to scan from
     * @param attributeNames {AWS.DynamoDB.DocumentClient.ExpressionAttributeNameMap} Map of attribute names
     * @param attributeValues {AWS.DynamoDB.DocumentClient.ExpressionAttributeValueMap} Map of attribute values
     * @param expression {string} Filter expression
     * @param attributesToReturn {string} Attributes to return. Default is ALL_ATTRIBUTES
     * @param lastEvaluatedKey {AWS.DynamoDB.DocumentClient.Key} LastEvaluatedKey of response. Supplied by recursion
     */
    ScanAsync(tableName, attributeNames, attributeValues, expression, attributesToReturn, lastEvaluatedKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${DynamoDBHelper.name}.${this.ScanAsync.name}`;
            this.TraceInputs(action, { tableName, attributeNames, attributeValues, expression, attributesToReturn, lastEvaluatedKey });
            // guard clauses
            if (this.IsNullOrEmpty(tableName)) {
                throw new Error(`[${action}]-Must supply tableName`);
            }
            if (this.IsNullOrEmpty(attributeNames)) {
                throw new Error(`[${action}]-Must supply attributeNames`);
            }
            if (this.IsNullOrEmpty(attributeValues)) {
                throw new Error(`[${action}]-Must supply attributeValues`);
            }
            if (this.IsNullOrEmpty(expression)) {
                throw new Error(`[${action}]-Must supply expression`);
            }
            // set defaults
            if (this.IsNullOrEmpty(attributesToReturn)) {
                attributesToReturn = 'ALL_ATTRIBUTES';
            }
            // create params object
            const params = {
                ExpressionAttributeNames: attributeNames,
                ExpressionAttributeValues: attributeValues,
                FilterExpression: expression,
                Select: attributesToReturn,
                TableName: tableName,
            };
            if (this.IsNullOrEmpty(lastEvaluatedKey)) {
                params.ExclusiveStartKey = lastEvaluatedKey;
            }
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.scan(params).promise();
            this.TraceResponse(action, response);
            // recursively call the function if LastEvaluatedKey is present
            if (response.LastEvaluatedKey && response.Items) {
                const nextParameters = yield this.ScanAsync(tableName, attributeNames, attributeValues, expression, attributesToReturn, response.LastEvaluatedKey);
                if (nextParameters.Items) {
                    for (const item of nextParameters.Items) {
                        response.Items.push(item);
                    }
                }
            }
            return response;
        });
    }
    /**
     * Update an item by key
     * @param tableName {string} Table name to update in
     * @param keyName {string} Name of key column
     * @param keyValue {string | number} Value of key column
     * @param attributeNames {AWS.DynamoDB.DocumentClient.ExpressionAttributeNameMap} Map of attribute names
     * @param attributeValues {AWS.DynamoDB.DocumentClient.ExpressionAttributeValueMap} Map of attribute values
     * @param conditionExpression {string} Condition expression
     * @param updateExpression {string} Update expression
     */
    UpdateByKeyAsync(tableName, keyName, keyValue, attributeNames, attributeValues, conditionExpression, updateExpression) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${DynamoDBHelper.name}.${this.UpdateByKeyAsync.name}`;
            this.TraceInputs(action, { tableName, keyName, keyValue, attributeNames, attributeValues, conditionExpression, updateExpression });
            // guard clauses
            if (this.IsNullOrEmpty(tableName)) {
                throw new Error(`[${action}]-Must supply tableName`);
            }
            if (this.IsNullOrEmpty(keyName)) {
                throw new Error(`[${action}]-Must supply keyName`);
            }
            if (this.IsNullOrEmpty(keyValue.toString())) {
                throw new Error(`[${action}]-Must supply keyValue`);
            }
            if (this.IsNullOrEmpty(attributeNames)) {
                throw new Error(`[${action}]-Must supply attributeNames`);
            }
            if (this.IsNullOrEmpty(attributeValues)) {
                throw new Error(`[${action}]-Must supply attributeValues`);
            }
            if (this.IsNullOrEmpty(conditionExpression)) {
                throw new Error(`[${action}]-Must supply conditionExpression`);
            }
            if (this.IsNullOrEmpty(updateExpression)) {
                throw new Error(`[${action}]-Must supply updateExpression`);
            }
            // create attribute map
            const attributeMap = {};
            attributeMap[keyName] = keyValue;
            // create params object
            const params = {
                ConditionExpression: conditionExpression,
                ExpressionAttributeNames: attributeNames,
                ExpressionAttributeValues: attributeValues,
                Key: attributeMap,
                TableName: tableName,
                UpdateExpression: updateExpression,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.update(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
}
exports.DynamoDBHelper = DynamoDBHelper;
//# sourceMappingURL=dynamodb.js.map