"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = require("aws-sdk");
const base_1 = require("./base");
/**
 * Lambda Helper
 */
class LambdaHelper extends base_1.BaseHelper {
    /**
     * Initializes new instance of LambdaHelper
     * @param logger {ILogger} Injected logger
     * @param repository {AWS.Lambda} Injected Repository. A new repository will be created if not supplied
     * @param options {AWS.Lambda.ClientConfiguration} Injected configuration if a Repository is supplied
     */
    constructor(logger, repository, options) {
        super(logger);
        /**
         * Default batch size for Dynamo Event Source Mapping
         */
        this.DefaultDynamoBatchSize = 100;
        /**
         * Default batch size for Kinesis Event Source Mapping
         */
        this.DefaultKinesisBatchSize = 100;
        /**
         * Default batch size for SQS Event Source Mapping
         */
        this.DefaultSQSBatchSize = 10;
        this.Repository = repository || new AWS.Lambda(options);
    }
    /**
     * Disable event source mapping for a Dynamo stream
     * @param functionName {string} Function name to update. Best to use the full ARN
     * @param uuid {string} Event source mapping uuid
     * @param batchSize {number} Batch size for the event source mapping. Default 100
     */
    DisableDynamoEventSourceMappingAsync(functionName, uuid, batchSize) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.UpdateEventSourceMappingAsync(functionName, false, uuid, batchSize || this.DefaultDynamoBatchSize);
        });
    }
    /**
     * Disable event source mapping for a Kinesis stream
     * @param functionName {string} Function name to update. Best to use the full ARN
     * @param uuid {string} Event source mapping uuid
     * @param batchSize {number} Batch size for the event source mapping. Default 100
     */
    DisableKinesisEventSourceMappingAsync(functionName, uuid, batchSize) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.UpdateEventSourceMappingAsync(functionName, false, uuid, batchSize || this.DefaultKinesisBatchSize);
        });
    }
    /**
     * Disable event source mapping for a SQS queue
     * @param functionName {string} Function name to update. Best to use the full ARN
     * @param uuid {string} Event source mapping uuid
     * @param batchSize {number} Batch size for the event source mapping. Default 10
     */
    DisableSQSEventSourceMappingAsync(functionName, uuid, batchSize) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.UpdateEventSourceMappingAsync(functionName, false, uuid, batchSize || this.DefaultSQSBatchSize);
        });
    }
    /**
     * Enable event source mapping for a Dynamo stream
     * @param functionName {string} Function name to update. Best to use the full ARN
     * @param uuid {string} Event source mapping uuid
     * @param batchSize {number} Batch size for the event source mapping. Default 100
     */
    EnableDynamoEventSourceMappingAsync(functionName, uuid, batchSize) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.UpdateEventSourceMappingAsync(functionName, true, uuid, batchSize || this.DefaultDynamoBatchSize);
        });
    }
    /**
     * Enable event source mapping for a Kinesis stream
     * @param functionName {string} Function name to update. Best to use the full ARN
     * @param uuid {string} Event source mapping uuid
     * @param batchSize {number} Batch size for the event source mapping. Default 100
     */
    EnableKinesisEventSourceMappingAsync(functionName, uuid, batchSize) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.UpdateEventSourceMappingAsync(functionName, true, uuid, batchSize || this.DefaultKinesisBatchSize);
        });
    }
    /**
     * Enable event source mapping for a SQS queue
     * @param functionName {string} Function name to update. Best to use the full ARN
     * @param uuid {string} Event source mapping uuid
     * @param batchSize {number} Batch size for the event source mapping. Default 10
     */
    EnableSQSEventSourceMappingAsync(functionName, uuid, batchSize) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.UpdateEventSourceMappingAsync(functionName, true, uuid, batchSize || this.DefaultSQSBatchSize);
        });
    }
    /**
     * Get an Event Source Mapping
     * @param uuid {string} Uuid of Event Source Mapping
     */
    GetEventSourceMappingAsync(uuid) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${LambdaHelper.name}.${this.GetEventSourceMappingAsync.name}`;
            this.TraceInputs(action, { uuid });
            // guard clauses
            if (this.IsNullOrEmpty(uuid)) {
                throw new Error(`[${action}]-Must supply uuid`);
            }
            // create params object
            const params = {
                UUID: uuid,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.getEventSourceMapping(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * List Event Source Mappings
     * @param functionName {string} Function name to get event source mappings for. Best to use the full ARN
     * @param eventSourceArn {string} Event Source ARN
     */
    ListEventSourceMappingsAsync(functionName, eventSourceArn) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${LambdaHelper.name}.${this.ListEventSourceMappingsAsync.name}`;
            this.TraceInputs(action, { functionName, eventSourceArn });
            // guard clauses
            if (this.IsNullOrEmpty(functionName)) {
                throw new Error(`[${action}]-Must supply functionName`);
            }
            if (this.IsNullOrEmpty(eventSourceArn)) {
                throw new Error(`[${action}]-Must supply eventSourceArn`);
            }
            // create params object
            const params = {
                EventSourceArn: eventSourceArn,
                FunctionName: functionName,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.listEventSourceMappings(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Invoke lambda async
     * @param functionName {string} Function name to invoke
     * @param payload {T} Payload to pass to function
     */
    InvokeAsync(functionName, payload) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${LambdaHelper.name}.${this.InvokeAsync.name}`;
            this.TraceInputs(action, { functionName, payload });
            // guard clauses
            if (this.IsNullOrEmpty(functionName)) {
                throw new Error(`[${action}]-Must supply functionName`);
            }
            // create params object
            const params = {
                FunctionName: functionName,
                InvokeArgs: payload,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.invokeAsync(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Invoke lambda sync
     * @param functionName {string} Function name to invoke
     * @param payload {T} Payload to pass to function
     */
    InvokeSync(functionName, payload) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${LambdaHelper.name}.${this.InvokeSync.name}`;
            this.TraceInputs(action, { functionName, payload });
            // guard clauses
            if (this.IsNullOrEmpty(functionName)) {
                throw new Error(`[${action}]-Must supply functionName`);
            }
            // create params object
            const params = {
                FunctionName: functionName,
                Payload: payload,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.invoke(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Update an event source mapping
     * @param functionName {string} Function name to update. Best to use the full ARN
     * @param enabled {boolean} Turn the event source mapping on or off
     * @param uuid {string} Event source mapping uuid
     * @param batchSize {number} Batch size for the event source mapping
     */
    UpdateEventSourceMappingAsync(functionName, enabled, uuid, batchSize) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${LambdaHelper.name}.${this.UpdateEventSourceMappingAsync.name}`;
            this.TraceInputs(action, { functionName, enabled, uuid, batchSize });
            // guard clauses
            if (this.IsNullOrEmpty(functionName)) {
                throw new Error(`[${action}]-Must supply functionName`);
            }
            if (this.IsNullOrEmpty(uuid)) {
                throw new Error(`[${action}]-Must supply uuid`);
            }
            // create params object
            const params = {
                BatchSize: batchSize,
                Enabled: enabled,
                FunctionName: functionName,
                UUID: uuid,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.updateEventSourceMapping(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
}
exports.LambdaHelper = LambdaHelper;
//# sourceMappingURL=lambda.js.map