"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = require("aws-sdk");
const base_1 = require("./base");
/**
 * S3 Helper
 */
class S3Helper extends base_1.BaseHelper {
    /**
     * Initializes new instance of S3Helper
     * @param logger {ILogger} Injected logger
     * @param repository {AWS.S3} Injected Repository. A new repository will be created if not supplied
     * @param options {AWS.S3.ClientConfiguration} Injected configuration if a Repository is supplied
     */
    constructor(logger, repository, options) {
        super(logger);
        this.Repository = repository || new AWS.S3(options);
    }
    /**
     * Copy an object from a source to a target
     * @param sourceBucket {string} Source bucket name
     * @param sourceKey {string} Source key
     * @param destinationBucket {string} Destination bucket name
     * @param destinationKey {string} Destination key
     */
    CopyObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.CopyObjectAsync.name}`;
            this.TraceInputs(action, { sourceBucket, sourceKey, destinationBucket, destinationKey });
            // guard clauses
            if (this.IsNullOrEmpty(sourceBucket)) {
                throw new Error(`[${action}]-Must supply sourceBucket`);
            }
            if (this.IsNullOrEmpty(sourceKey)) {
                throw new Error(`[${action}]-Must supply sourceKey`);
            }
            if (this.IsNullOrEmpty(destinationBucket)) {
                throw new Error(`[${action}]-Must supply destinationBucket`);
            }
            if (this.IsNullOrEmpty(destinationKey)) {
                throw new Error(`[${action}]-Must supply destinationKey`);
            }
            // create params object
            const params = {
                Bucket: destinationBucket,
                CopySource: `${sourceBucket}/${sourceKey}`,
                Key: destinationKey,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.copyObject(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Create a S3 bucket
     * @param name {string} Bucket name
     */
    CreateBucketAsync(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.CreateBucketAsync.name}`;
            this.TraceInputs(action, { name });
            // guard clauses
            if (this.IsNullOrEmpty(name)) {
                throw new Error(`[${action}]-Must supply name`);
            }
            // create params object
            const params = {
                Bucket: name,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.createBucket(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Delete a S3 bucket
     * @param name {string} Bucket name
     */
    DeleteBucketAsync(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteBucketAsync.name}`;
            this.TraceInputs(action, { name });
            // guard clauses
            if (this.IsNullOrEmpty(name)) {
                throw new Error(`[${action}]-Must supply name`);
            }
            // create params object
            const params = {
                Bucket: name,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.deleteBucket(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Delete an object
     * @param bucket {string} Bucket name
     * @param key {string} Object key to delete
     */
    DeleteObjectAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteObjectAsync.name}`;
            this.TraceInputs(action, { bucket, key });
            // guard clauses
            if (this.IsNullOrEmpty(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.IsNullOrEmpty(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // create params object
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.deleteObject(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Delete multiple objects
     * @param bucket {string} Bucket name
     * @param keys {string[]} Array of object keys to delete
     */
    DeleteObjectsAsync(bucket, keys) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteObjectsAsync.name}`;
            this.TraceInputs(action, { bucket, keys });
            // guard clauses
            if (this.IsNullOrEmpty(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (!keys || keys.length === 0) {
                throw new Error(`[${action}]-Must supply at least one key`);
            }
            // create array of ObjectIdentifiers
            const keysArray = [];
            for (const key of keys) {
                keysArray.push({ Key: key });
            }
            // create params object
            const params = {
                Bucket: bucket,
                Delete: { Objects: keysArray },
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.deleteObjects(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Get a JSON typed object from S3
     * @param bucket {string} Bucket to retrieve from
     * @param key {string} File prefix and name
     */
    GetObjectAsJsonAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = (yield this.GetObjectAsync(bucket, key)).Body;
            const buffer = data ? data : undefined;
            const json = buffer ? buffer.toString() : '';
            return JSON.parse(json);
        });
    }
    /**
     * Get an object from S3
     * @param bucket {string} Bucket to retrieve from
     * @param key {string} File prefix and name
     */
    GetObjectAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetObjectAsync.name}`;
            this.TraceInputs(action, { bucket, key });
            // guard clauses
            if (this.IsNullOrEmpty(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.IsNullOrEmpty(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // create params object
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.getObject(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
    /**
     * Upload a file to S3 Bucket
     * @param bucket {string} Bucket to upload to
     * @param key {string} File prefix and name
     * @param body {string} File contents
     * @param acl {S3CannedACL} S3Canned ACL. Default is 'bucket-owner-full-control'
     * @param encoding {string} File encoding. Default is 'utf-8'
     * @returns Promise<string> - URL of uploaded file
     */
    PutObjectAsync(bucket, key, body, acl, encoding) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.PutObjectAsync.name}`;
            this.TraceInputs(action, { bucket, key, body, acl });
            // guard clauses
            if (this.IsNullOrEmpty(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.IsNullOrEmpty(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // set defaults
            if (this.IsNullOrEmpty(acl)) {
                acl = 'bucket-owner-full-control';
            }
            if (this.IsNullOrEmpty(encoding)) {
                encoding = 'utf-8';
            }
            // create params object
            const params = {
                ACL: acl,
                Body: body,
                Bucket: bucket,
                ContentEncoding: encoding,
                Key: key,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.putObject(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
}
exports.S3Helper = S3Helper;
//# sourceMappingURL=s3.js.map