"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = require("aws-sdk");
const base_1 = require("./base");
/**
 * SES Helper
 */
class SESHelper extends base_1.BaseHelper {
    /**
     * Initializes new instance of SESHelper
     * @param logger {ILogger} Injected logger
     * @param repository {AWS.SES} Injected Repository. A new repository will be created if not supplied
     * @param options {AWS.SES.ClientConfiguration} Injected configuration if a Repository is supplied
     */
    constructor(logger, repository, options) {
        super(logger);
        this.Repository = repository || new AWS.SES(options);
    }
    /**
     * Send an email
     * @param subject {string} Subject of email
     * @param toAddresses {string[]} Array of addresses to send to
     * @param fromAddress {string} Address to send from
     * @param body {string | Buffer} HTML body of message
     */
    SendEmailAsync(subject, toAddresses, fromAddress, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${SESHelper.name}.${this.SendEmailAsync.name}`;
            this.TraceInputs(action, { subject, toAddresses, fromAddress, body });
            // guard clauses
            if (this.IsNullOrEmpty(subject)) {
                throw new Error(`[${action}]-Must supply subject`);
            }
            if (this.IsNullOrEmpty(toAddresses)) {
                throw new Error(`[${action}]-Must supply toAddresses`);
            }
            if (this.IsNullOrEmpty(fromAddress)) {
                throw new Error(`[${action}]-Must supply fromAddress`);
            }
            // create params object
            const params = {
                Destination: {
                    ToAddresses: toAddresses,
                },
                Message: {
                    Body: {
                        Html: {
                            Data: body.toString(),
                        },
                    },
                    Subject: {
                        Data: subject,
                    },
                },
                Source: fromAddress,
            };
            this.TraceRequest(action, params);
            // make AWS call
            const response = yield this.Repository.sendEmail(params).promise();
            this.TraceResponse(action, response);
            return response;
        });
    }
}
exports.SESHelper = SESHelper;
//# sourceMappingURL=ses.js.map